<?php
/**
* Classes de validao de dados de entrada
* @author DigitalDev
* @version 01.00.00 B
*/
class DDException extends Exception
{
	
}


class DDDataValidation
{
	private $v_data_values;
	private $v_data_itens_object;

	//constantes de charset
	const UTF_8 = 1;
	const ISO_8859_1 = 2;
	
	//constantes para formato de data
	const DATE_EN = 1;
	const DATE_BR = 2;
	
	//constantes de erro
	const ErrorEmpty = 1;
	const ErrorNumeric = 2;
	const ErrorMaximum = 3;
	const ErrorMinimum = 4;
	const ErrorCPF = 5;
	const ErrorEmail = 6;
	const ErrorDate = 7;
	
	function __construct()
	{
		$this->v_data_values = array();
		$this->v_data_itens_object = array();
	}
	
	/**
	* Adiciona dados para ser validados
	* 
	* @param string $st_alias - Alias definido pelo desenvolvedor para fazer referencia ao dado
	* @param mixed $mx_data - Dado a ser validado
	*/
	public function setData($st_alias,$st_value)
	{
		$this->v_data_values[$st_alias] = $st_value;
		$this->v_data_itens_object[$st_alias] = new DDDataValidationItem($st_value);
	}
	
	/**
	* Retorna um objeto contendo dados a serem validados
	* 
	* @param string $st_alias - Alias definido pelo desenvolvedor para fazer referencia ao dado
	* @return DDDataValidationItem
	* @throws DDException
	*/
	public function getData($st_alias)
	{
		if(isset($this->v_data_itens_object[$st_alias]))
			return $this->v_data_itens_object[$st_alias];
		else
			return new DDDataValidationItem();
	}

	/**
	* Retorna os dados sem nenhuma modificao
	* 
	* @param string $st_alias - Alias definido pelo desenvolvedor para fazer referencia aos dados
	* @throws DDException
	*/
	public function getOriginal($st_alias)
	{
		if(isset($v_campos[$st_alias]))
			return $v_data_values[$st_alias];
		else
			throw new DDException('Alias '.$st_alias.' not found');
	}
	
	/**
 	* Retorna a quantidade de erros, idenpendente
 	* do alias referente ao campo
 	* 
 	* @return integer
	*/
	public function countErrors()
	{
		foreach($this->v_data_itens_object AS $data_objects)
			$in_errors += $data_objects->countErrors();
		return $in_errors;
	}
}


class DDDataValidationItem
{
	private $mx_value;
	private $v_error;
	private $v_boolean_error;
	
	/**
	* O valor do dado pode ser definidos ou no
	* 
	* @param mixed $mx_value
	*/
	function __construct($mx_value = null)
	{
		if($mx_value != null)
			$this->set($mx_value);
	}
	
	
	/**
	* Recebe o dado  ser validado
	* 
	* @param mixed $mx_value
	*/
	public function set($mx_value)
	{
		$this->mx_value = $mx_value;
	}
	
	/**
	* Retorna o dado validado
	* 
	* @return mixed
	*/
	public function get()
	{
		return $this->mx_value;
	}
	
	/**
	* Retorna, se houver, o erro referente ao alias de verficao
	* 
	* @param flag $mx_erro_code
	* 	Use DDDataValidation::ErrorEmpty - retorna o erro de preenchimento do campo
	*		DDDataValidation::ErrorNumeric - retorna o erro da validao de numero
	*		DDDataValidation::ErrorMaximum - retorna o erro da validao de quantidade de caracteres
	*		DDDataValidation::ErrorMinimum - retorna o erro da validao de quantidade de caracteres
	*		DDDataValidation::ErrorCPF - retorna o erro da validao de CPF 
	*		DDDataValidation::ErrorEmail - retorna o erro da validao de e-mail
	*		DDDataValidation::ErrorDate - retornar o erro da validao de data	
	*
	* @return string
	*/
	public function getErrorMsg($mx_erro_code)
	{
		if(isset($this->v_error[$mx_erro_code]))
			return $this->v_error[$mx_erro_code];
	}
	
	/**
	* Retorna o ltimo erro ocorrido
	* 
	* @return string
	*/
	public function getLastErrorMsg()
	{
		$st_message = end($this->v_error);
		if(isset($st_message))
			return $st_message;
	}
	
	
	/**
	* Retorna o primeiro erro ocorrido
	* 
	* @return string
	*/
	public function getFirstErrorMsg()
	{
		if(isset($this->v_error))
		{
			$st_message = reset($this->v_error);
			if(isset($st_message))
				return $st_message;
		}	
	}
	
	
	/**
	* Retorna positivo se houver erro e nega
	*/
	public function getError()
	{
		if(isset($this->v_boolean_error[$mx_erro_code]))
			return $this->v_boolean_error[$mx_erro_code];
	}
	
	/**
	* Retorna todos os erros, separados apenas pelo delimitador
	* 
	* @param character $ch_delimiter - Delimitador
	* @return string
	*/
	public function getAllErrosMessages($ch_delimiter = ' - ')
	{
		foreach($this->v_error AS $st_error)
			if(isset($st_all_error))
				$st_all_error .= $ch_delimiter.$st_error;
			else
				$st_all_error .= $st_error;
		return $st_all_error;	
	}
	
	/**
	* Retorna a quantidade de erros gerados para aquele campo
	* 
	* @return integer
	*/
	public function countErrors()
	{
		return count($this->v_error);
	}
	
	
	/************************/
	/* Mtodos de restrio */
	/************************/
	
	/**
	* Verifica se o dado passado est vazio
	* 
	* @param string $st_message - Mensagem de erro
	* @param mixed $mx_error_code - Cdigo de erro, pode ser personalizado
	* @return DDDataValidationItem
	*/
	public function isEmpty( $st_message = null, $mx_error_code = DDDataValidation::ErrorEmpty )
	{
		$mx_value = $this->trim()->get();
		if(!(strlen($mx_value) > 0))
		{
			if($st_message != null)
				$this->v_error[$mx_error_code] = $st_message;
			$this->v_boolean_error[$mx_error_code] = TRUE;
		}		
		return $this;	
	}
	
	/**
	* Verifica se o dado passado  um numero
	* 
	* @param string $st_message - Mensagem de erro
	* @param mixed $mx_error_code - Cdigo de erro, pode ser personalizado
	* @return DDDataValidationItem
	*/
	public function isNumeric( $st_message = null, $mx_error_code = DDDataValidation::ErrorNumeric )
	{
		$mx_value = $this->trim()->get();
		$mx_value = str_replace(',', '.', $mx_value);
		if(!(is_numeric($mx_value)))
		{
			if($st_message != null)
				$this->v_error[$mx_error_code] = $st_message;
			$this->v_boolean_error[$mx_error_code] = TRUE;
		}
		return $this;
	}
	
	/**
	* Verifica se o dado passado tem quantidade de
	* caracteres acima do limite passado como parametro
	* 
	* @param integer $in_max_caracter_count - Quantidade mxima de caracteres
	* @param string $st_message - Mensagem de erro
	* @param mixed $mx_error_code - Cdigo de erro, pode ser personalizado
	* @return DDDataValidationItem
	* @throws DDException
	*/
	public function isMaximum( $in_max_caracter_count , $st_message = null , $mx_error_code = DDDataValidation::ErrorMaximum )
	{
		if(is_int($in_max_caracter_count))
		{
			if((strlen($this->mx_value) >= $in_max_caracter_count))
			{
				if($st_message != null)
					$this->v_error[$mx_error_code] = $st_message;
				$this->v_boolean_error[$mx_error_code] = TRUE;
			}
		}
		else
			throw new DDException('the variable in_max_caracter_count isn\'t numeric');
		return $this;			
	}
	
	/**
	* Verifica se o dado passado tem quantidade de
	* caracteres abaixo do limite passado como parametro
	* 
	* @param integer $in_min_caracter_count - Quantidade mnima de caracteres
	* @param string $st_message - Mensagem de erro
	* @param mixed $mx_error_code - Cdigo de erro, pode ser personalizado
	* @return DDDataValidationItem
	* @throws DDException
	*/
	public function isMinimum( $in_min_caracter_count , $st_message = null , $mx_error_code = DDDataValidation::ErrorMinimum )
	{
		if(is_numeric($in_min_caracter_count))
		{
			if(strlen($this->mx_value) <= $in_min_caracter_count)
			{
				if($st_message != null)
					$this->v_error[$mx_error_code] = $st_message;
				$this->v_boolean_error[$mx_error_code] = TRUE;
			}
		}
		else
			throw new DDException('the variable in_min_caracter_count isn\'t numeric');
		return $this;
	}
	
	/**
	* Verifica se o dado  um CPF vlido
	* 
	* @param string $st_message - Mensagem de erro
	* @param mixed $mx_error_code - Cdigo de erro, pode ser personalizado
	* @return DDDataValidationItem
	*/
	public function isCPF( $st_message , $mx_error_code = DDDataValidation::ErrorCPF )
	{
		$st_cpf = $this->mx_value;
		$st_cpf = str_pad(ereg_replace('[^0-9]', '', $st_cpf), 11, '0', STR_PAD_LEFT);
		if 
		(
			strlen($st_cpf) != 11 ||
			$st_cpf == '00000000000' ||
			$st_cpf == '11111111111' ||
			$st_cpf == '22222222222' ||
			$st_cpf == '33333333333' ||
			$st_cpf == '44444444444' ||
			$st_cpf == '55555555555' ||
			$st_cpf == '66666666666' ||
			$st_cpf == '77777777777' ||
			$st_cpf == '88888888888' ||
			$st_cpf == '99999999999'
		)
			$this->v_error[$mx_error_code] = $st_message;
		else
			for ($t = 9; $t < 11; $t++)
			{ 
				for ($d = 0, $c = 0; $c < $t; $c++) 
					$d += $st_cpf{$c} * (($t + 1) - $c);
				
	
				$d = ((10 * $d) % 11) % 10;	
				if ($st_cpf{$c} != $d)
				{
					if($st_message != null)
						$this->v_error[$mx_error_code] = $st_message;
					$this->v_boolean_error[$mx_error_code] = TRUE;
				}	
			}	
		return $this;
	}
	
	/**
	* Verifica se o dado  um e-mail vlido
	* 
	* @param string $st_message - Mensagem de erro
	* @param mixed $mx_error_code - Cdigo de erro, pode ser personalizado
	* @return DDDataValidationItem
	*/
	public function isEmail( $st_message , $mx_error_code = DDDataValidation::ErrorEmail )
	{
		$st_email = $this->mx_value;
		$bo_valid = eregi
		(
			"^[0-9a-z]([-_.]?[0-9a-z])*@[0-9a-z]([-.]?[0-9a-z])*\\.[a-z]{2,3}$",
			$st_email,
			$check
		); 
		
		if(!$bo_valid)
		{
			if($st_message != null)
				$this->v_error[$mx_error_code] = $st_message;
			$this->v_boolean_error[$mx_error_code] = TRUE;
		}
		return $this;
	}
	
	/**
	* Verifica se o dado  uma data vlida
	* @param integer $in_format - Usado para definir qual o formato da data
	* 		Use DDDataValidation::DATE_BR para formato DD/MM/AAAA
	* 		Use DDDataValidation::DATE_EN para formato AAAA/MM/DD
	* 
	* @param string $st_message - Mensagem de erro
	* @param mixed $mx_error_code - Cdigo de erro, pode ser personalizado
	* @return DDDataValidationItem
	* @throws DDException
	*/
	public function isDate(  $st_message = null , $in_format = DDDataValidation::DATE_BR , $mx_error_code = DDDataValidation::ErrorDate )
	{
		$dt_date = $this->mx_value;
		$v_date = preg_split('/[[:punct:]]/',$dt_date);		
		if( !is_numeric($v_date[0]) || !is_numeric($v_date[1]) || !is_numeric($v_date[2])  )
		{
			{
					if($st_message != null)
						$this->v_error[$mx_error_code] = $st_message;
					$this->v_boolean_error[$mx_error_code] = TRUE;
			}
			return $this;
		}
		
		if($in_format == DDDataValidation::DATE_BR)
		{
			$in_day = $v_date[0];
			$in_month =  $v_date[1];
			$in_year =  $v_date[2];
		}
		elseif($in_format == DDDataValidation::DATE_EN)
		{
			$in_day = $v_date[2];
			$in_month =  $v_date[1];
			$in_year =  $v_date[0];
		}
		else
			throw new DDException('Invalid value for variable $in_format');
		
		if (!checkdate( $in_month , $in_day , $in_year ))		
			$this->v_error[$mx_error_code] = $st_message;
		return $this;
	}
	
	/*********************/
	/* Mtodos de filtro */
	/*********************/
	
	/**
	* Retira, por padro, caracteres de espao ( ), tab (\t), retorno de carro (\r)
	* e quebra de linhas (\n) no inicio e no fim do dado. Pode ser personalizada 
	* usando a varivel $st_characters
	* 
	* @param string $st_characters - Caracteres que devem ser retirados no inicio
	* 		e no fim do dado, o valor padro  " \t\n\r"
	* 
	* @return DDDataValidationItem
	*/
	public function trim( $st_characters = " \t\n\r" )
	{
		$this->mx_value = trim( $this->mx_value , $st_characters );
		return $this;
	}
	
	/**
	* Retira, por padro, todas as tags HTML encontradas no dado
	* 
	* @param string $allowable_tags - Use-a para permitir tags especificadas 
	* @return DDDataValidationItem
	*/
	public function noHTML( $allowable_tags = '' )
	{
		$this->mx_value = strip_tags($this->mx_value,$allowable_tags);
		return $this;
	}
	
	/**
	* Retira tudo que no for uma letra do alfabeto 
	* ou um numero de 0 a 9 do dado
	* 
	* @return DDDataValidationItem
	*/
	public function alphaNum()
	{
		$this->trim();
		$this->noHTML();
		$this->mx_value = ereg_replace("([[:punct:]]| )","",$this->mx_value);
		return $this;
	}
	
	/**
	* Retira tudo que no for um numero do dado
	* 
	* @return DDDataValidationItem
	*/
	public function numeric()
	{
		$this->trim();
		$this->noHTML();
		$this->mx_value = ereg_replace("([[:punct:]]|[[:alpha:]]| )","",$this->mx_value);
		return $this;
	}
	
	/************************/
	/* Mtodos de converso */
	/************************/
	
	
	/**
	* Converte uma data do formato Brasileiro Ex: 10/11/2011
	* 	para o formato Americano Ex: 2011/11/10 ou vice-versa
	* 
	* @param integer $f_actual_format - Especificar o formato atual da data
	* 	Use DDDataValidation::DATE_BR para formato Brasileiro
	* 	Use DDDataValidation::DATE_EN para formato Americano
	* @param integer $f_to_format - Especificar o formato de data desejado
	* 	Use DDDataValidation::DATE_BR para formato Brasileiro
	* 	Use DDDataValidation::DATE_EN para formato Americano
	* @param character $ch_delimiter - Especificar o caractere de separao
	* 	para dia ms e ano
	* @throws DDException
	*/
	public function convertDate($f_actual_format = DDDataValidation::DATE_BR , $f_to_format = DDDataValidation::DATE_EN , $ch_delimiter = '-' )
	{
		$dt_date = $this->mx_value;
		$v_date = preg_split('/[[:punct:]]/',$dt_date);
		
		if( !is_numeric($v_date[0]) || !is_numeric($v_date[1]) || !is_numeric($v_date[2])  )
			throw new DDException('this data isn\'t a valid date');
		
		if($f_actual_format != DDDataValidation::DATE_BR && $f_actual_format != DDDataValidation::DATE_EN)
			throw new DDException('$f_actual_format isn\'t a valid flag');
		
		if($f_to_format != DDDataValidation::DATE_BR && $f_to_format != DDDataValidation::DATE_EN)
			throw new DDException('$f_to_format isn\'t a valid flag');
		
		if($f_actual_format == DDDataValidation::DATE_BR)
		{
			$in_day = $v_date[0];
			$in_month =  $v_date[1];
			$in_year =  $v_date[2];
		}
		elseif($f_actual_format == DDDataValidation::DATE_EN)
		{
			$in_day = $v_date[2];
			$in_month =  $v_date[1];
			$in_year =  $v_date[0];
		}
	
		
		if (!checkdate( $in_month , $in_day , $in_year ))
			throw new DDException('this data isn\'t a valid date');
		
		
		if($f_to_format == DDDataValidation::DATE_BR)
			$this->mx_value = $in_day . $ch_delimiter . $in_month . $ch_delimiter . $in_year;			
		elseif($f_to_format == DDDataValidation::DATE_EN)
			$this->mx_value = $in_year . $ch_delimiter . $in_month . $ch_delimiter . $in_day;				
		return $this;
	}
	
	
	/**
	* Converte todos os caractres especiais em suas
	* respectivas entidades HTML
	* 
	* @return DDDataValidationItem
	*/
	public function htmlEntities()
	{
		$this->mx_value = htmlentities($this->mx_value);
		return $this;
	}

	/**
	* Converte entidades HTML em seus respectivos
	* caracteres especiais
	* 
	* @return DDDataValidationItem
	*/
	public function htmlEntitiesDecode()
	{
		$this->mx_value = html_entity_decode($this->mx_value);
		return $this;
	}
	
	/**
	* Converte characteres de final de linha (\n) para
	* para a tag <br/> do HTML
	* 
	* @return DDDataValidationItem
	*/
	public function nlToBr()
	{
		$this->mx_value = nl2br($this->mx_value);
		return $this;
	}
	

	/**
	* seta o charset do dado
	* 
	* @param integer $in_actual_charset - Seta o charset atual do dado
	* 	Use DDDataValidation::ISO_8859_1 para ISO-8859-1
	* 	Use DDDataValidation::UTF-8 para UTF-8
	* @param integer $in_to_charset - Define o charset desejado
	* 	Use DDDataValidation::ISO_8859_1 para ISO-8859-1
	* 	Use DDDataValidation::UTF-8 para UTF-8
	* @return DDDataValidationItem
	* @throws DDException
	*/
	public function setCharset($in_actual_charset = DDDataValidation::ISO_8859_1 , $in_to_charset = DDDataValidation::UTF_8)
	{
		if($in_actual_charset != DDDataValidation::ISO_8859_1 && $in_actual_charset != DDDataValidation::UTF_8)
			throw new DDException('invalid actual charset variable'); 
		
		if($in_to_charset != DDDataValidation::ISO_8859_1 && $in_to_charset != DDDataValidation::UTF_8)
			throw new DDException('invalid to charset variable');
		
		if($in_actual_charset == DDDataValidation::UTF_8 && $in_to_charset == DDDataValidation::ISO_8859_1)
			$this->mx_value = utf8_decode($this->mx_value);
		
		if($in_actual_charset == DDDataValidation::ISO_8859_1 && $in_to_charset == DDDataValidation::UTF_8)
			$this->mx_value = utf8_encode($this->mx_value);
		return $this;
	}
}
?>
